@@ -22,7 +22,7 @@ use Unicode::Normalize;
 use parent qw(Class::Accessor);
 __PACKAGE__->follow_best_practice;
 
-our $VERSION = '2.12';
+our $VERSION = '2.13';
 our $BETA_VERSION = 0; # Is this a beta version?
 
 our $logger  = Log::Log4perl::get_logger('main');
@@ -58,10 +58,6 @@ $CONFIG->{state}{uniqignore} = {};
 $CONFIG->{state}{crossrefkeys} = {};
 $CONFIG->{state}{xrefkeys} = {};
 
-# Set tracking, parent->child and child->parent
-$CONFIG->{state}{set}{pc} = {};
-$CONFIG->{state}{set}{cp} = {};
-
 # Citekeys which refer to the same entry
 $CONFIG->{state}{citkey_aliases} = {};
 
@@ -74,6 +70,7 @@ $CONFIG->{state}{xdata} = [];
 $CONFIG->{state}{graph} = {};
 
 $CONFIG->{state}{seenkeys} = {};
+$CONFIG->{globalstate}{seenkeys} = {};
 
 # Track the order of keys as cited. Keys cited in the same \cite*{} get the same order
 # Used for sorting schemes which use \citeorder
@@ -100,8 +97,6 @@ sub _init {
   $CONFIG->{state}{datafiles} = [];
   $CONFIG->{state}{crossref} = [];
   $CONFIG->{state}{xdata} = [];
-  $CONFIG->{state}{set}{pc} = {};
-  $CONFIG->{state}{set}{cp} = {};
 
   return;
 }
@@ -146,9 +141,14 @@ sub _initopts {
   # There is a special default config file for tool mode
   # Referring to as yet unprocessed cmd-line tool option as it isn't processed until below
   if ($opts->{tool}) {
-    (my $vol, my $dir, undef) = File::Spec->splitpath( $INC{"Biber/Config.pm"} );
-    $dir =~ s/\/$//; # splitpath sometimes leaves a trailing '/'
-    _config_file_set(File::Spec->catpath($vol, "$dir", 'biber-tool.conf'));
+    if (my $bc = $opts->{configtool}) { # Only used in tests to use source-tree biber-tool.conf
+      _config_file_set($bc);
+    }
+    else {
+      (my $vol, my $dir, undef) = File::Spec->splitpath( $INC{"Biber/Config.pm"} );
+      $dir =~ s/\/$//; # splitpath sometimes leaves a trailing '/'
+      _config_file_set(File::Spec->catpath($vol, "$dir", 'biber-tool.conf'));
+    }
   }
 
   # Normal user config file - overrides tool mode defaults
@@ -158,7 +158,7 @@ sub _initopts {
   # This has to go after _config_file_set() as this is what defines option scope
   # in tool mode (from the .conf file)
   foreach (keys %CONFIG_DEFAULT_BIBLATEX) {
-    Biber::Config->setblxoption($_, $CONFIG_DEFAULT_BIBLATEX{$_});
+    Biber::Config->setblxoption(0, $_, $CONFIG_DEFAULT_BIBLATEX{$_});
   }
 
   # Command-line overrides everything else
@@ -349,6 +349,7 @@ sub _config_file_set {
                                                             qr/\Aconstraints\z/,
                                                             qr/\Aconstraint\z/,
                                                             qr/\Aentrytype\z/,
+                                                            qr/\Aentryfields\z/,
                                                             qr/\Adatetype\z/,
                                                             qr/\Acondition\z/,
                                                             qr/\A(?:or)?filter\z/,
@@ -372,6 +373,10 @@ sub _config_file_set {
     my $scope = $bcfscopeopts->{type};
     foreach my $bcfscopeopt ($bcfscopeopts->{option}->@*) {
       my $opt = $bcfscopeopt->{content};
+      $CONFIG_BIBLATEX_OPTIONS{$scope}{$opt}{OUTPUT} = $bcfscopeopt->{backendout} || 0;
+      if (my $bin = Biber::Utils::process_backendin($bcfscopeopt->{backendin})) {
+        $CONFIG_BIBLATEX_OPTIONS{$scope}{$opt}{INPUT} = $bin;
+      }
       $CONFIG_OPTSCOPE_BIBLATEX{$opt}{$scope} = 1;
       $CONFIG_SCOPEOPT_BIBLATEX{$scope}{$opt} = 1;
     }
@@ -415,10 +420,10 @@ sub _config_file_set {
       foreach my $t ($v->@*) {
         my $latype = $t->{type};
         if ($latype eq 'global') {
-          Biber::Config->setblxoption('labelalphatemplate', $t);
+          Biber::Config->setblxoption(0, 'labelalphatemplate', $t);
         }
         else {
-          Biber::Config->setblxoption('labelalphatemplate',
+          Biber::Config->setblxoption(0, 'labelalphatemplate',
                                       $t,
                                       'ENTRYTYPE',
                                       $latype);
@@ -439,7 +444,7 @@ sub _config_file_set {
 
         }
         $lants->{$t->{name}} = $lant;
-        Biber::Config->setblxoption('labelalphanametemplate', $lants);
+        Biber::Config->setblxoption(0, 'labelalphanametemplate', $lants);
       }
     }
     elsif (lc($k) eq 'uniquenametemplate') {
@@ -454,7 +459,7 @@ sub _config_file_set {
         }
         $unts->{$unt->{name}} = $untval;
       }
-      Biber::Config->setblxoption('uniquenametemplate', $unts);
+      Biber::Config->setblxoption(0, 'uniquenametemplate', $unts);
     }
     elsif (lc($k) eq 'sortingnamekeytemplate') {
       my $snss;
@@ -482,15 +487,15 @@ sub _config_file_set {
         }
         $snss->{$sns->{name}} = $snkps;
       }
-      Biber::Config->setblxoption('sortingnamekeytemplate', $snss);
+      Biber::Config->setblxoption(0, 'sortingnamekeytemplate', $snss);
     }
     elsif (lc($k) eq 'transliteration') {
       foreach my $tr ($v->@*) {
         if ($tr->{entrytype}[0] eq '*') { # already array forced for another option
-          Biber::Config->setblxoption('translit', $tr->{translit});
+          Biber::Config->setblxoption(0, 'translit', $tr->{translit});
         }
         else {                  # per_entrytype
-          Biber::Config->setblxoption('translit',
+          Biber::Config->setblxoption(0, 'translit',
                                       $tr->{translit},
                                       'ENTRYTYPE',
                                       $tr->{entrytype}[0]);
@@ -530,7 +535,7 @@ sub _config_file_set {
       Biber::Config->setconfigfileoption($k, $sms);
     }
     elsif (lc($k) eq 'inheritance') {# This is a biblatex option
-      Biber::Config->setblxoption($k, $v);
+      Biber::Config->setblxoption(0, $k, $v);
     }
     elsif (lc($k) eq 'sortexclusion') {# This is a biblatex option
       foreach my $sex ($v->@*) {
@@ -538,7 +543,7 @@ sub _config_file_set {
         foreach my $ex ($sex->{exclusion}->@*) {
           $excludes->{$ex->{content}} = 1;
         }
-        Biber::Config->setblxoption('sortexclusion',
+        Biber::Config->setblxoption(0, 'sortexclusion',
                                     $excludes,
                                     'ENTRYTYPE',
                                     $sex->{type});
@@ -550,7 +555,7 @@ sub _config_file_set {
         foreach my $in ($sin->{inclusion}->@*) {
           $includes->{$in->{content}} = 1;
         }
-        Biber::Config->setblxoption('sortinclusion',
+        Biber::Config->setblxoption(0, 'sortinclusion',
                                     $includes,
                                     'ENTRYTYPE',
                                     $sin->{type});
@@ -561,11 +566,11 @@ sub _config_file_set {
       foreach my $presort ($v->@*) {
         # Global presort default
         unless (exists($presort->{type})) {
-          Biber::Config->setblxoption('presort', $presort->{content});
+          Biber::Config->setblxoption(0, 'presort', $presort->{content});
         }
         # Per-type default
         else {
-          Biber::Config->setblxoption('presort',
+          Biber::Config->setblxoption(0, 'presort',
                                       $presort->{content},
                                       'ENTRYTYPE',
                                       $presort->{type});
@@ -577,10 +582,10 @@ sub _config_file_set {
       foreach my $ss ($v->@*) {
         $sorttemplates->{$ss->{name}} = Biber::_parse_sort($ss);
       }
-      Biber::Config->setblxoption('sortingtemplate', $sorttemplates);
+      Biber::Config->setblxoption(0, 'sortingtemplate', $sorttemplates);
     }
     elsif (lc($k) eq 'datamodel') {# This is a biblatex option
-      Biber::Config->setblxoption('datamodel', $v);
+      Biber::Config->addtoblxoption(0, 'datamodel', $v);
     }
     elsif (exists($v->{content})) { # simple option
       Biber::Config->setconfigfileoption($k, $v->{content});
@@ -884,6 +889,21 @@ sub isexplicitoption {
 #################################
 
 
+=head2 addtoblxoption
+
+    Add to an array global biblatex option
+
+=cut
+
+sub addtoblxoption {
+  shift; # class method so don't care about class name
+  my ($secnum, $opt, $val) = @_;
+  if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{GLOBAL}) {
+    push $CONFIG->{options}{biblatex}{GLOBAL}{$opt}->@*, $val;
+  }
+  return;
+}
+
 =head2 setblxoption
 
     Set a biblatex option on the appropriate scope
@@ -892,14 +912,19 @@ sub isexplicitoption {
 
 sub setblxoption {
   shift; # class method so don't care about class name
-  my ($opt, $val, $scope, $scopeval) = @_;
+  my ($secnum, $opt, $val, $scope, $scopeval) = @_;
   if (not defined($scope)) { # global is the default
-    if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}->{GLOBAL}) {
+    if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{GLOBAL}) {
       $CONFIG->{options}{biblatex}{GLOBAL}{$opt} = $val;
     }
   }
-  else { # Per-type/entry options need to specify type/entry too
-    if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}->{$scope}) {
+  elsif ($scope eq 'ENTRY') {
+    if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{$scope}) {
+      $CONFIG->{options}{biblatex}{$scope}{$scopeval}{$secnum}{$opt} = $val;
+    }
+  }
+  else {
+    if ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{$scope}) {
       $CONFIG->{options}{biblatex}{$scope}{$scopeval}{$opt} = $val;
     }
   }
@@ -910,30 +935,33 @@ sub setblxoption {
 
     Get a biblatex option from the global, per-type or per entry scope
 
-    getblxoption('option', ['entrytype'], ['citekey'])
+    getblxoption('secnum', 'option', ['entrytype'], ['citekey'])
 
     Returns the value of option. In order of decreasing preference, returns:
     1. Biblatex option defined for entry
     2. Biblatex option defined for entry type
     3. Biblatex option defined globally
 
+    section number needs to be present only for per-entry options as these might
+    differ between sections
+
 =cut
 
 sub getblxoption {
   no autovivification;
   shift; # class method so don't care about class name
-  my ($opt, $entrytype, $citekey) = @_;
+  my ($secnum, $opt, $entrytype, $citekey) = @_;
   if ( defined($citekey) and
-       $CONFIG_OPTSCOPE_BIBLATEX{$opt}->{ENTRY} and
-       defined $CONFIG->{options}{biblatex}{ENTRY}{$citekey}{$opt}) {
-    return $CONFIG->{options}{biblatex}{ENTRY}{$citekey}{$opt};
+       $CONFIG_OPTSCOPE_BIBLATEX{$opt}{ENTRY} and
+       defined $CONFIG->{options}{biblatex}{ENTRY}{$citekey}{$secnum}{$opt}) {
+    return $CONFIG->{options}{biblatex}{ENTRY}{$citekey}{$secnum}{$opt};
   }
   elsif (defined($entrytype) and
-         $CONFIG_OPTSCOPE_BIBLATEX{$opt}->{ENTRYTYPE} and
+         $CONFIG_OPTSCOPE_BIBLATEX{$opt}{ENTRYTYPE} and
          defined $CONFIG->{options}{biblatex}{ENTRYTYPE}{lc($entrytype)}{$opt}) {
     return $CONFIG->{options}{biblatex}{ENTRYTYPE}{lc($entrytype)}{$opt};
   }
-  elsif ($CONFIG_OPTSCOPE_BIBLATEX{$opt}->{GLOBAL}) {
+  elsif ($CONFIG_OPTSCOPE_BIBLATEX{$opt}{GLOBAL}) {
     return $CONFIG->{options}{biblatex}{GLOBAL}{$opt};
   }
 }
@@ -948,8 +976,8 @@ sub getblxoption {
 sub getblxentryoptions {
   no autovivification;
   shift; # class method so don't care about class name
-  my $key = shift;
-  return keys $CONFIG->{options}{biblatex}{ENTRY}{$key}->%*;
+  my ($secnum, $key) = @_;
+  return keys $CONFIG->{options}{biblatex}{ENTRY}{$key}{$secnum}->%*;
 }
 
 ##############################
@@ -1011,89 +1039,6 @@ sub get_graph {
   return $CONFIG->{state}{graph}{$type};
 }
 
-=head2 set_set_pc
-
-  Record a parent->child set relationship
-
-=cut
-
-sub set_set_pc {
-  shift; # class method so don't care about class name
-  my ($parent, $child) = @_;
-  $CONFIG->{state}{set}{pc}{$parent}{$child} = 1;
-  return;
-}
-
-=head2 set_set_cp
-
-  Record a child->parent set relationship
-
-=cut
-
-sub set_set_cp {
-  shift; # class method so don't care about class name
-  my ($child, $parent) = @_;
-  $CONFIG->{state}{set}{cp}{$child}{$parent} = 1;
-  return;
-}
-
-=head2 get_set_pc
-
-  Return a boolean saying if there is a parent->child set relationship
-
-=cut
-
-sub get_set_pc {
-  shift; # class method so don't care about class name
-  my ($parent, $child) = @_;
-  return exists($CONFIG->{state}{set}{pc}{$parent}{$child}) ? 1 : 0;
-}
-
-=head2 get_set_cp
-
-  Return a boolean saying if there is a child->parent set relationship
-
-=cut
-
-sub get_set_cp {
-  shift; # class method so don't care about class name
-  my ($child, $parent) = @_;
-  return exists($CONFIG->{state}{set}{cp}{$child}{$parent}) ? 1 : 0;
-}
-
-=head2 get_set_children
-
-  Return a list of children for a parent set
-
-=cut
-
-sub get_set_children {
-  shift; # class method so don't care about class name
-  my $parent = shift;
-  if (exists($CONFIG->{state}{set}{pc}{$parent})) {
-    return (keys $CONFIG->{state}{set}{pc}{$parent}->%*);
-  }
-  else {
-    return ();
-  }
-}
-
-=head2 get_set_parents
-
-  Return a list of parents for a child of a set
-
-=cut
-
-sub get_set_parents {
-  shift; # class method so don't care about class name
-  my $child = shift;
-  if (exists($CONFIG->{state}{set}{cp}{$child})) {
-    return (keys $CONFIG->{state}{set}{cp}{$child}->%*);
-  }
-  else {
-    return ();
-  }
-}
 
 
 =head2 set_inheritance
@@ -1222,11 +1167,7 @@ sub get_seenkey {
     return $CONFIG->{state}{seenkeys}{$section}{$key};
   }
   else {
-    my $count;
-    foreach my $section (keys $CONFIG->{state}{seenkeys}->%*) {
-      $count += $CONFIG->{state}{seenkeys}{$section}{$key};
-    }
-    return $count;
+    return $CONFIG->{globalstate}{seenkeys}{$key};
   }
 }
 
@@ -1242,6 +1183,7 @@ sub incr_seenkey {
   my $key = shift;
   my $section = shift;
   $CONFIG->{state}{seenkeys}{$section}{$key}++;
+  $CONFIG->{globalstate}{seenkeys}{$key}++;
   return;
 }
 
@@ -1381,7 +1323,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2018 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2019 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
